@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__  = "$Id: focus_tracking_presenter.py 3957 2008-06-04 16:43:04Z richb $"
-__version__   = "$Revision: 3957 $"
-__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
+__id__  = "$Id: focus_tracking_presenter.py 3994 2008-06-23 13:25:32Z wwalker $"
+__version__   = "$Revision: 3994 $"
+__date__      = "$Date: 2008-06-23 09:25:32 -0400 (Mon, 23 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -609,23 +609,29 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # that are now showing (such as gnome-screensaver-dialog).
                     # See bug #530368 for more details.
                     #
+                    # Also, we might be running into a gnome-panel
+                    # applet, which is indicated by a host application
+                    # with no children.  See bug #536985.
+                    #
                     eType = event.type
                     if (eType == "window:activate") \
                        or ((eType.startswith("focus")) 
                          and (event.source.getRole() == pyatspi.ROLE_FRAME)) \
                        or (eType.startswith("object:state-changed:showing")
                          and (event.source.getRole() == pyatspi.ROLE_PANEL)
-                         and state.contains(pyatspi.STATE_MODAL)):
+                         and state.contains(pyatspi.STATE_MODAL)) \
+                       or (event.host_application \
+                           and len(event.host_application) == 0):
 
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
-                        debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
-                                      + orca_state.activeScript.name)
 
                         self.setActiveScript(
                             self.getScript(event.host_application or \
                                               event.source.getApplication()))
+                        debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
+                                      + orca_state.activeScript.name)
 
                         # Load in the application specific settings for the
                         # app for this event (if found).
@@ -923,12 +929,14 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         Returns True if the event should be consumed.
         """
 
+        consume = False
         if orca_state.activeScript \
            and orca_state.activeScript.consumesKeyboardEvent(keyboardEvent):
-            self._enqueueEvent(keyboardEvent)
-            return True
-        else:
-            return False
+            consume = not orca_state.bypassNextCommand
+            if consume:
+                self._enqueueEvent(keyboardEvent)
+
+        return consume
 
     def processBrailleEvent(self, brailleEvent):
         """Called whenever a cursor key is pressed on the Braille display.
