@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 3975 2008-06-12 19:08:52Z wwalker $"
-__version__   = "$Revision: 3975 $"
-__date__      = "$Date: 2008-06-12 15:08:52 -0400 (Thu, 12 Jun 2008) $"
+__id__        = "$Id: orca.py 4016 2008-07-01 17:40:13Z wwalker $"
+__version__   = "$Revision: 4016 $"
+__date__      = "$Date: 2008-07-01 13:40:13 -0400 (Tue, 01 Jul 2008) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -607,7 +607,9 @@ def keyEcho(event):
 
 def _setClickCount(inputEvent):
     """Sets the count of the number of clicks a user has made to one
-    of the non-modifier keys on the keyboard.
+    of the non-modifier keys on the keyboard.  Note that this looks at
+    the event_string (keysym) instead of hw_code (keycode) because
+    the Java platform gives us completely different keycodes for keys.
 
     Arguments:
     - inputEvent: the current input event.
@@ -621,7 +623,7 @@ def _setClickCount(inputEvent):
         orca_state.clickCount = 0
     elif not isinstance(lastInputEvent, KeyboardEvent):
         orca_state.clickCount = 1
-    elif (lastInputEvent.hw_code != inputEvent.hw_code) or \
+    elif (lastInputEvent.event_string != inputEvent.event_string) or \
          (lastInputEvent.modifiers != inputEvent.modifiers):
         orca_state.clickCount = 1
     elif (inputEvent.time - lastInputEvent.time) < \
@@ -796,6 +798,11 @@ def _processKeyboardEvent(event):
                     exitLearnMode(keyboardEvent)
 
                 consumed = True
+
+            if not consumed \
+               and not isModifierKey(keyboardEvent.event_string) \
+               and keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
+                orca_state.bypassNextCommand = False
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
@@ -1332,6 +1339,13 @@ def usage():
 
     print "-v, --version                %s" % platform.version
 
+    # Translators: this is a testing option for the command line.  It prints
+    # the names of the applications known to the accessibility infrastructure
+    # to stdout and then exits.
+    #
+    print "-l, --list-apps              " + \
+          _("Print the known running applications")
+
     # Translators: this is the description of the command line option
     # '-s, --setup, --gui-setup' that will initially display a GUI dialog
     # that would allow the user to set their Orca preferences.
@@ -1479,7 +1493,7 @@ def main():
         #
         opts, args = getopt.getopt(
             arglist,
-            "?stnvd:e:u:",
+            "?stnvld:e:u:",
             ["help",
              "user-prefs-dir=",
              "enable=",
@@ -1488,6 +1502,7 @@ def main():
              "gui-setup",
              "text-setup",
              "no-setup",
+             "list-apps",
              "version"])
         for opt, val in opts:
             if opt in ("-u", "--user-prefs-dir"):
@@ -1545,6 +1560,13 @@ def main():
             if opt in ("-v", "--version"):
                 print "Orca %s" % platform.version
                 abort(0)
+            if opt in ("-l", "--list-apps"):
+                apps = filter(lambda x: x is not None,
+                              pyatspi.Registry.getDesktop(0))
+                for app in apps:
+                    print app.name
+                abort(0)
+
     except:
         debug.printException(debug.LEVEL_OFF)
         usage()
