@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Each script maintains a set of key bindings, braille bindings, and
 AT-SPI event listeners.  The key bindings are an instance of
@@ -33,10 +33,10 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py 3561 2008-02-11 18:11:45Z wwalker $"
-__version__   = "$Revision: 3561 $"
-__date__      = "$Date: 2008-02-11 13:11:45 -0500 (Mon, 11 Feb 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: script.py 3997 2008-06-23 17:18:28Z joanied $"
+__version__   = "$Revision: 3997 $"
+__date__      = "$Date: 2008-06-23 13:18:28 -0400 (Mon, 23 Jun 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import braillegenerator
@@ -46,6 +46,7 @@ import keybindings
 import orca_state
 import settings
 import speechgenerator
+import structural_navigation
 import where_am_I
 import bookmarks
 
@@ -76,6 +77,7 @@ class Script:
         #
         self.presentIfInactive = True
 
+        self.structuralNavigation = self.getStructuralNavigation()
         self.inputEventHandlers = {}
         self.pointOfReference = {}
         self.setupInputEventHandlers()
@@ -184,6 +186,27 @@ class Script:
         """
         return speechgenerator.SpeechGenerator(self)
 
+    def getEnabledStructuralNavigationTypes(self):
+        """Returns a list of the structural navigation object types
+        enabled in this script.
+        """
+        return []
+
+    def getStructuralNavigation(self):
+        """Returns the 'structural navigation' class for this script.
+        """
+        types = self.getEnabledStructuralNavigationTypes()
+        return structural_navigation.StructuralNavigation(self, types)
+
+    def useStructuralNavigationModel(self):
+        """Returns True if we should use structural navigation. Most
+        scripts will have no need to override this.  Gecko does however
+        because within an HTML document there are times when we do want
+        to use it and times when we don't even though it is enabled,
+        e.g. in a form field.
+        """
+        return self.structuralNavigation.enabled
+
     def getWhereAmI(self):
         """Returns the "where am I" class for this script.
         """
@@ -329,16 +352,26 @@ class Script:
 
         user_bindings = None
         user_bindings_map = settings.keyBindingsMap
-        if user_bindings_map.has_key(self.__module__):
+        if self.__module__ in user_bindings_map:
             user_bindings = user_bindings_map[self.__module__]
-        elif user_bindings_map.has_key("default"):
+        elif "default" in user_bindings_map:
             user_bindings = user_bindings_map["default"]
 
         consumes = False
         if user_bindings:
-            consumes = user_bindings.getInputHandler(keyboardEvent) != None
+            handler = user_bindings.getInputHandler(keyboardEvent)
+            if handler \
+                 and handler.function in self.structuralNavigation.functions:
+                return self.useStructuralNavigationModel()
+            else:
+                consumes = handler != None
         if not consumes:
-            consumes = self.keyBindings.getInputHandler(keyboardEvent) != None
+            handler = self.keyBindings.getInputHandler(keyboardEvent)
+            if handler \
+                 and handler.function in self.structuralNavigation.functions:
+                return self.useStructuralNavigationModel()
+            else:
+                consumes = handler != None
         return consumes
 
     def processKeyboardEvent(self, keyboardEvent):
@@ -373,9 +406,9 @@ class Script:
         user_bindings = None
 
         user_bindings_map = settings.keyBindingsMap
-        if user_bindings_map.has_key(self.__module__):
+        if self.__module__ in user_bindings_map:
             user_bindings = user_bindings_map[self.__module__]
-        elif user_bindings_map.has_key("default"):
+        elif "default" in user_bindings_map:
             user_bindings = user_bindings_map["default"]
 
         consumed = False
@@ -397,17 +430,17 @@ class Script:
         """
         user_bindings = None
         user_bindings_map = settings.brailleBindingsMap
-        if user_bindings_map.has_key(self.__module__):
+        if self.__module__ in user_bindings_map:
             user_bindings = user_bindings_map[self.__module__]
-        elif user_bindings_map.has_key("default"):
+        elif "default" in user_bindings_map:
             user_bindings = user_bindings_map["default"]
 
         command = brailleEvent.event
         consumes = False
         if user_bindings:
-            consumes = user_bindings.has_key(command)
+            consumes = command in user_bindings
         if not consumes:
-            consumes = self.brailleBindings.has_key(command)
+            consumes = command in self.brailleBindings
         return consumes
 
     def processBrailleEvent(self, brailleEvent):
@@ -444,16 +477,16 @@ class Script:
         command = brailleEvent.event
 
         user_bindings_map = settings.brailleBindingsMap
-        if user_bindings_map.has_key(self.name):
+        if self.name in user_bindings_map:
             user_bindings = user_bindings_map[self.name]
-        elif user_bindings_map.has_key("default"):
+        elif "default" in user_bindings_map:
             user_bindings = user_bindings_map["default"]
 
-        if user_bindings and user_bindings.has_key(command):
+        if user_bindings and command in user_bindings:
             handler = user_bindings[command]
             consumed = handler.processInputEvent(self, brailleEvent)
 
-        if (not consumed) and self.brailleBindings.has_key(command):
+        if (not consumed) and command in self.brailleBindings:
             handler = self.brailleBindings[command]
             consumed = handler.processInputEvent(self, brailleEvent)
 
